/* *************************************************
 * Title: Pathfinder Demo
 * Author: Phaelax
 *
 * Left mouse button fills in impassable tiles, right
 * mouse button clears thems. Press 'A' and 'B' to
 * set the source and target points, respectively.
 * The in-game menu will show you which number to
 * press to call the appropriate search method.
 * *************************************************
*/



setVirtualResolution(640,480)


// These 3 lines are needed to use the library
#include "pathfinder.agc"

gosub SEARCH_Declare_UDT

initPathfinder()




// Define some variables for this demo
#constant MAPWIDTH   40
#constant MAPHEIGHT  30
#constant CELLWIDTH  16
#constant CELLHEIGHT 16

global SourceX as integer
global SourceY as integer
global TargetX as integer
global TargetY as integer
global time = 0
global diags = 0
global found = 0


// Create a map and initialize 1 path
CreateSearchMap(MAPWIDTH, MAPHEIGHT)
CreateSearchPathLists(0)


setPrintSize(14)
setPrintColor(255,0,0)
do



    mapControls()



    rem starting point and target point
    drawBox(SourceX*CELLWIDTH,SourceY*CELLHEIGHT,SourceX*CELLWIDTH+CELLWIDTH,SourceY*CELLHEIGHT+CELLHEIGHT, 0,255,0)
    drawBox(TargetX*CELLWIDTH,TargetY*CELLHEIGHT,TargetX*CELLWIDTH+CELLWIDTH,TargetY*CELLHEIGHT+CELLHEIGHT, 255,0,0)



    // output instructions and junk
    if diags = 0
        td$ = "off"
    else
        td$ = "on"
    endif
    print("(1) A*8    (2) A*4    (3) Flood8    (4) Flood4")
    print("(D) Restrict Diagonals is "+td$)
    print("")
    print("Moves Taken: "+str(GetSearchPathSize(0)))
    if found = 0
        print("Search Failed: "+str(time)+" ms")
    else
        print("Search Successful: "+str(time)+" ms")
        drawPath(0)
    endif


    // draw map
    drawMap()

    Sync()
loop






/* ******************************************************
 * ******************************************************
 * ******************************************************
 * ******************************************************
 * ******************************************************
 * ******************************************************
 * ******************************************************
 */



function ProcessAStar8()
    start = getMilliseconds()
    found = searchMapAStar8(0, SourceX, SourceY, TargetX, TargetY)
    time = getMilliseconds() - start
endfunction found


function ProcessAStar4()
    start = getMilliseconds()
    found = SearchMapAStar4(0, SourceX, SourceY, TargetX, TargetY)
    time = getMilliseconds() - start
endfunction found


function ProcessFlood4()
    start = getMilliseconds()
    found = SearchMapFlood4(0, SourceX, SourceY, TargetX, TargetY)
    time = getMilliseconds() - start
endfunction found


function ProcessFlood8()
    start = getMilliseconds()
    found = SearchMapFlood8(0, SourceX, SourceY, TargetX, TargetY)
    time = getMilliseconds() - start
endfunction found



function mapControls()
    x = getPointerX() / CELLWIDTH
    y = getPointerY() / CELLHEIGHT

    if getRawMouseLeftState() = 1
        setSearchMap(x, y, 1)
    endif
    if getRawMouseRightState() = 1
        setSearchMap(x, y, 0)
    endif

    if getRawKeyPressed(65) = 1
        SourceX = x
        SourceY = y
    endif

    if getRawKeyPressed(66) = 1
        TargetX = x
        TargetY = y
    endif

    if getRawKeyPressed(68) = 1
        diags = 1 - diags
        SetSearchRestrictDiagonals(diags)
    endif

    if getRawKeyPressed(49) = 1 then found = ProcessAStar8()
    if getRawKeyPressed(50) = 1 then found = ProcessAStar4()
    if getRawKeyPressed(51) = 1 then found = ProcessFlood8()
    if getRawKeyPressed(52) = 1 then found = ProcessFlood4()

endfunction





function drawMap()
    for x = 0 to MAPWIDTH
        for y = 0 to MAPHEIGHT
             if GetSearchMap(x,y) > 0
                drawBox(x*CELLWIDTH,y*CELLHEIGHT,x*CELLWIDTH+CELLWIDTH,y*CELLHEIGHT+CELLHEIGHT,255,255,255)
            endif
        next y
    next x


    // draw grid lines
    for x = 1 to MAPWIDTH-1
        drawLine(x*CELLWIDTH,0,x*CELLWIDTH,480,64,64,64)
    next x
    for y = 1 to MAPHEIGHT-1
        drawLine(0,y*CELLHEIGHT,640,y*CELLHEIGHT,64,64,64)
    next y
endfunction




function drawPath(path)
    for i = 1 to GetSearchPathSize(path)-1
        x = getSearchPathX(path, i)
        y = getSearchPathY(path, i)
        drawBox(x*CELLWIDTH,y*CELLHEIGHT,x*CELLWIDTH+CELLWIDTH,y*CELLHEIGHT+CELLHEIGHT,40,80,255)
    next i
endfunction


function drawBox(x1, y1, x2, y2, r, g, b)
    for x = x1 to x2
        drawLine(x,y1,x,y2, r,g,b)
    next x
endfunction

